import java.util.Arrays;

public class MergeSort {

   private Integer[] data;    // values to be sorted
   private int low, high;     // to be sorted: data[low..high)
   private Integer[] scratch; // scratch space for merging

   MergeSort (Integer[] data, int low, int high) {
     this.data = data;
     this.low = low;
     this.high = high;
     scratch = new Integer[data.length];
   }

   public void sort() {
     if (low >= high - 1)  // size <= 1: sorted
       return;
       // mid point
     int mid = low + (high - low)/2;
       // left and right halves
     MergeSort left = new MergeSort(data, low, mid);
     MergeSort right = new MergeSort(data, mid, high);
     left.sort();  // recursion on left
     right.sort(); // recursion on right
     merge(mid);   // merge sorted halves
   }

   private void merge(int mid) {
     int left = low, right = mid, k = low;
     while (left < mid && right < high) {
       if (data[left] <= data[right]) {
         scratch[k] = data[left];
         left++;
       } else {
         scratch[k] = data[right];
         right++;
       }
       k++;
     }
     while (left < mid) scratch[k++] = data[left++];
     while (right < high) scratch[k++] = data[right++];
     for (int i = low; i < high; i++)
       data[i] = scratch[i];
   }

   @Override
   public String toString() {
     return Arrays.toString(data);
   }

   public static void main(String[] args) {
      Integer[] toSort = new Integer[] {10,-3,2,1,87,3,3,5,6,-7};
      MergeSort sorter = new MergeSort(toSort, 0, toSort.length);
      sorter.sort();
      System.out.println(sorter);
   }

}
   
